/*
  Web Server
 
 Circuit:
 * Ethernet shield attached to pins 10, 11, 12, 13
 * Analog inputs attached to pins A0 through A5 (optional)
 
 */

#include <SD.h>
#include <SPI.h>
#include <Ethernet.h>

const int refresh = 5;     // time to refresh (s)
const int led2 = 2;        // the number of the LED pin
const int led3 = 3;        // the number of the LED pin
boolean button = false;    // the number of the pushbutton pin

// change this to match your SD shield or module;
// Arduino Ethernet shield: pin 4
// Adafruit SD shields and modules: pin 10
// Sparkfun SD shield: pin 8
const int chipSelect = 4;    

// the media access control (ethernet hardware) address for the shield:
byte mac[] = { 0x90, 0xA2, 0xDA, 0x0F, 0x2D, 0x48 };
//the IP address for the shield:
byte ip[] = { 192, 168, 0, 14 };    
// the router's gateway address:
byte gateway[] = { 10, 0, 0, 1 };
// the subnet:
byte subnet[] = { 255, 255, 0, 0 };

// Initialize the Ethernet server library
// with the IP address and port you want to use 
// (port 80 is default for HTTP):
EthernetServer server(80);
EthernetClient client;

void setup()
{
  // initialize the LED pins as input / output:
  pinMode(led2, INPUT);
  pinMode(led3, OUTPUT);
  
  // Open serial communications and wait for port to open:
  Serial.begin(9600);

  // wait for serial port to connect. Needed for Leonardo only
  while ( !Serial );

  Serial.print("Initializing SD card...");
  // On the Ethernet Shield, CS is pin 4. It's set as an output by default.
  // Note that even if it's not used as the CS pin, the hardware SS pin 
  // (10 on most Arduino boards, 53 on the Mega) must be left as an output 
  // or the SD library functions will not work. 
   pinMode(10, OUTPUT);
   
  if ( !SD.begin(chipSelect) ) {
    Serial.println("initialization failed!");
    return;
    }
  Serial.println("initialization done.");

  // start the Ethernet connection and the server:
  Serial.print("Start ethernet connection...");
  Ethernet.begin(mac, ip);  // IP adressé par DHCP
// Ethernet.begin(mac, ip, gateway, subnet);

  // start listening for clients
  server.begin();
  
  Serial.print("server is at "); Serial.println(Ethernet.localIP());
}

void loop()
{
  // listen for incoming clients
  client = server.available();
  if ( client ) {
    Serial.println("new client");
    
    // an http request ends with a blank line
    String  getRequest = "";
            
    while ( client.connected() ) {
      if ( client.available() ) {
        char c = client.read();
        Serial.write(c);  // pour debug
        
        if ( c != '\n' )
          getRequest += c;
        else {
           if ( getRequest.startsWith("GET") )
               traiterRequete(getRequest.c_str());
           getRequest = "";
           }
       
        // if you've gotten to the end of the line (received a newline
        // character) and the line is blank, the http request has ended,
        // so you can send a reply
        if ( c == '\n' ) {
          // send a standard http response header
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println("Connection: close");                 // the connection will be closed after completion of the response
	  client.print("Refresh: "); client.println(refresh);  // refresh the page automatically
          client.println();
          
          // lecture code html (carte SD)
          readFile("index.htm");
          break;
          }        
        }
      }
      
    // give the web browser time to receive the data
    delay(refresh * 100);
    
    // close the connection:
    client.stop();
    Serial.println("client disconnected");
    }
}

void readFile(const char *_file)
{
  // open the file for reading:
  File myFile = SD.open(_file);
  
  if ( myFile ) {
    // read from the file until there's nothing else in it:
    while ( myFile.available() ) {
      char c = myFile.read();
      switch ( c ) {
        case -1:  // pas de caractère
          break;
          
        case '~':  // variable ~n à remplacer par la valeur lue sur la pin n° n
          while ( (c = myFile.read()) == -1);
          // lecture t°
          client.print( analogRead(c) );
          break;
          
        case '$':  // variable $n à remplacer par la valeur lue sur la pin n° n
          while ( (c = myFile.read()) == -1);
          // indicateur chauffage
          if ( c == '2' )
            client.print(digitalRead(led2) ? "on" : "off");
          // commande chauffage
          else if ( c == '3' ) {
            client.print(button ? "ON" : "OFF");
            digitalWrite(led3, button ? HIGH : LOW);
            }
          break;
          
        default:  // on affiche le code html
          client.print(c);
          }
      }

    // close the file:
    myFile.close();
    }
  else {
    // if the file didn't open, print an error:
    Serial.print("error opening "); Serial.println(_file);
    }
}

void traiterRequete(const char *request)
{
  Serial.print("Requete formulaire : "); Serial.println(request);
  
  String requete = request;
  int _start = requete.indexOf("/?");
  int _stop  = requete.indexOf('=');
  int value  = requete.indexOf(' ', _stop+1);
  
  // check if the pushbutton is pressed.
  // if it is, the button is HIGH:
  button = ( requete.substring(_stop+1, value) == "OFF" );
}

